# Generated by Django 4.2.13 on 2025-01-23 00:58

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('moses', '0001_initial'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='customuser',
            name='last_phone_number_confirmation_pins_sent',
        ),
        migrations.RemoveField(
            model_name='customuser',
            name='password_reset_code_sms_sent_at',
        ),
        migrations.RemoveField(
            model_name='customuser',
            name='phone_number_confirmation_code_sms_sent_at',
        ),
        migrations.AddField(
            model_name='customuser',
            name='password_reset_code_sms_unlocks_at',
            field=models.DateTimeField(blank=True, null=True, verbose_name='Password reset code unlocks at'),
        ),
        migrations.AddField(
            model_name='customuser',
            name='phone_number_candidate_confirmation_code_sms_unlocks_at',
            field=models.DateTimeField(blank=True, null=True, verbose_name='Phone number candidate confirmation code unlocks at'),
        ),
        migrations.AddField(
            model_name='customuser',
            name='phone_number_confirmation_code_sms_unlocks_at',
            field=models.DateTimeField(blank=True, null=True, verbose_name='Phone number confirmation code unlocks at'),
        ),
    ]
